#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include <stack>
using namespace std;

typedef long (*op_func)( long, long );

class opers {                              // реализуемые вычислителем операции
   static map< string, op_func > ops;  
public:
   op_func find( string& s ) {             // поиск операции по её изображению 
      try { return ops.at( s ); }
      catch(...) { return NULL; }          // исключение если операция не найдена
   }
} aops;
map< string, op_func > opers::ops = {  
   { "<", []( long o1, long o2 )-> long { return o1 < o2 ? 1 : 0; } },
   { ">", []( long o1, long o2 )-> long { return o1 > o2 ? 1 : 0; } },
   { "+", []( long o1, long o2 )-> long { return o1 + o2; } },
   { "-", []( long o1, long o2 )-> long { return o1 - o2; } },
   { "*", []( long o1, long o2 )-> long { return o1 * o2; } },
   { "/", []( long o1, long o2 )-> long { return o1 / o2; } },
   { "^", []( long o1, long o2 )-> long { 
                   if( 0 == o2 ) return 1;
                   long res = o1; 
                   while( o2-- > 1 ) res *= o1;
                   return res; } 
            }
};

long exec( vector<string>& p ) {          // вычисление выражения (если числовое)
   stack<long> stak;
   for( string &o : p ) {
      op_func eval = aops.find( o ); 
      if( eval ) {                         // операция
         if( stak.size() < 2 ) 
         throw syn_exception( "пустой стек операндов" );
         long o1, o2;
         o2 = stak.top();
         stak.pop();
         o1 = stak.top();
         stak.pop();
         stak.push( eval( o1, o2 ) );
      }
      else {                              // операнд
         try { 
            long n = stol( o );
            stak.push( n ); 
         }
         catch(...) {
            throw syn_exception( "не известный символ: " + o );
         }
      }
   }
   if( stak.size() != 1 )
      throw syn_exception( "не освобождённый стек операндов" );
   return stak.top();
}

istream& operator >>( istream& inp, string& line ) {
   do {
      getline( inp, line );
      if( '#' == line[ 0 ] ) continue;  // комментарий 
      if( !inp || line.empty() )        // завершение
         return inp;
      break;
   } while( true );
   cout << line << " => ";
   str2ppn pp; 
   try {                                // преобразовать
      vector<string> v = pp.convert( line );
      cout << pp;
   }
   catch( syn_exception& e ) {          // синтаксическая ошибка
      return inp;
   }
   try { 
      cout << " = " << exec( pp.outstr ) << endl; 
   }
   catch( syn_exception& e ) {  
      cout << endl << string( (const char*)e ) << endl;
   }
   return inp;
}

int main( int argc, char **argv ) {
   string text;
   if( argc > 1 ) {                     // читать из файла
      ifstream fin;
      fin.open( argv[ 1 ] );
      if( !fin ) {
         cerr << "Ошибочное имя файла!" << endl;
         return 1;
      }
      while( true ) {
         fin >> text;
         if( !fin || text.empty() ) break;
      }
      fin.close();
   }
   else {                               // читать с терминала
      cout << "Вводите выражения: " << endl;
      while( true ) {
         cout << "> ";
         cin >> text;
         if( !cin || text.empty() ) break;
      }
   }
}
